@@ -1,21 +1,21 @@
 import json
 import textwrap
 import uuid
-from typing import Dict, List, Optional, Union
+from collections import defaultdict
+from typing import DefaultDict, Dict, List, Union
 
-from aqueduct.dag import DAG
+from aqueduct.backend.response_models import (
+    GetWorkflowResponse,
+    SavedObjectUpdate,
+    WorkflowDagResponse,
+    WorkflowDagResultResponse,
+)
 from aqueduct.error import InvalidUserActionException, InvalidUserArgumentException
-from aqueduct.integrations.airflow_integration import AirflowIntegration
-from pydantic import BaseModel
+from aqueduct.flow_run import FlowRun
+from aqueduct.models.dag import DAG
+from aqueduct.utils.utils import format_header_for_print, generate_ui_url, parse_user_supplied_id
 
-from aqueduct import api_client
-
-from .enums import ArtifactType, OperatorType
-from .flow_run import FlowRun
-from .logger import logger
-from .operators import OperatorSpec, ParamSpec
-from .responses import WorkflowDagResponse, WorkflowDagResultResponse
-from .utils import format_header_for_print, generate_ui_url, parse_user_supplied_id
+from aqueduct import globals
 
 
 class Flow:
@@ -37,6 +37,20 @@ class Flow:
         """Returns the id of the flow."""
         return uuid.UUID(self._id)
 
+    def _get_workflow_resp(self) -> GetWorkflowResponse:
+        resp = globals.__GLOBAL_API_CLIENT__.get_workflow(self._id)
+        if len(resp.workflow_dag_results) == 0:
+            raise InvalidUserActionException("This flow has not been run yet.")
+        return resp
+
+    def name(self) -> str:
+        """Returns the latest name of the flow."""
+        resp = self._get_workflow_resp()
+        latest_result = resp.workflow_dag_results[-1]
+        latest_workflow_dag = resp.workflow_dags[latest_result.workflow_dag_id]
+        assert latest_workflow_dag.metadata.name is not None
+        return latest_workflow_dag.metadata.name
+
     def list_runs(self, limit: int = 10) -> List[Dict[str, str]]:
         """Lists the historical runs associated with this flow, sorted chronologically from most to least recent.
 
@@ -51,7 +65,7 @@ class Flow:
         if not isinstance(limit, int) or limit < 0:
             raise InvalidUserArgumentException("Limit must be a positive integer.")
 
-        resp = api_client.__GLOBAL_API_CLIENT__.get_workflow(self._id)
+        resp = globals.__GLOBAL_API_CLIENT__.get_workflow(self._id)
         return [
             dag_result.to_readable_dict()
             for dag_result in list(reversed(resp.workflow_dag_results))[:limit]
@@ -68,44 +82,7 @@ class Flow:
             metadata=dag_resp.metadata,
         )
 
-        # Because parameters are not stored in the db, we cannot trust its value in the operator spec.
-        # Instead, we'll need to fetch the parameter's value from the parameter operator's output.
-        param_artifacts = dag.list_artifacts(filter_to=[ArtifactType.PARAM])
-        for param_artifact in param_artifacts:
-            param_val = api_client.__GLOBAL_API_CLIENT__.get_artifact_result_data(
-                str(dag_result.id),
-                str(param_artifact.id),
-            )
-
-            # Skip the parameter update if the parameter was never computed.
-            if len(param_val) == 0:
-                logger().warning(
-                    "The parameter %s was not successfully computed. If you triggered this flow run with custom "
-                    "parameters, those parameter values will not be reflected in `FlowRun.describe()."
-                    % param_artifact.name
-                )
-                continue
-
-            dag.update_operator_spec(
-                # this works because the parameter op and artifact currently share the same name.
-                param_artifact.name,
-                OperatorSpec(
-                    param=ParamSpec(
-                        val=param_val,
-                    ),
-                ),
-            )
-
-        # Because the serialized functions are stored separately from the dag,
-        # We need to fetch them to complete the construction of the dag.
-        for operator in dag.list_operators(
-            filter_to=[OperatorType.CHECK, OperatorType.FUNCTION, OperatorType.METRIC]
-        ):
-            serialized_function = api_client.__GLOBAL_API_CLIENT__.export_serialized_function(
-                operator
-            )
-            dag.update_operator_function(operator, serialized_function)
-
+        # The dags for fetched flow runs are missing their serialized functions.
         return FlowRun(
             flow_id=self._id,
             run_id=str(dag_result.id),
@@ -116,10 +93,7 @@ class Flow:
         )
 
     def latest(self) -> FlowRun:
-        resp = api_client.__GLOBAL_API_CLIENT__.get_workflow(self._id)
-        if len(resp.workflow_dag_results) == 0:
-            raise InvalidUserActionException("This flow has not been run yet.")
-
+        resp = self._get_workflow_resp()
         latest_result = resp.workflow_dag_results[-1]
         latest_workflow_dag = resp.workflow_dags[latest_result.workflow_dag_id]
         return self._construct_flow_run(latest_result, latest_workflow_dag)
@@ -127,10 +101,7 @@ class Flow:
     def fetch(self, run_id: Union[str, uuid.UUID]) -> FlowRun:
         run_id = parse_user_supplied_id(run_id)
 
-        resp = api_client.__GLOBAL_API_CLIENT__.get_workflow(self._id)
-        assert (
-            len(resp.workflow_dag_results) > 0
-        ), "Every flow must have at least one run attached to it."
+        resp = self._get_workflow_resp()
 
         result = None
         for candidate_result in resp.workflow_dag_results:
@@ -146,9 +117,21 @@ class Flow:
         workflow_dag = resp.workflow_dags[result.workflow_dag_id]
         return self._construct_flow_run(result, workflow_dag)
 
+    def list_saved_objects(self) -> DefaultDict[str, List[SavedObjectUpdate]]:
+        """Get everything saved by the flow.
+
+        Returns:
+            A dictionary mapping the integration id to the list of table names/storage path.
+        """
+        workflow_objects = globals.__GLOBAL_API_CLIENT__.list_saved_objects(self._id).object_details
+        object_mapping = defaultdict(list)
+        for item in workflow_objects:
+            object_mapping[item.integration_name].append(item)
+        return object_mapping
+
     def describe(self) -> None:
         """Prints out a human-readable description of the flow."""
-        resp = api_client.__GLOBAL_API_CLIENT__.get_workflow(self._id)
+        resp = self._get_workflow_resp()
         latest_result = resp.workflow_dag_results[-1]
         latest_workflow_dag = resp.workflow_dags[latest_result.workflow_dag_id]
 
@@ -156,7 +139,7 @@ class Flow:
         assert latest_metadata.schedule is not None, "A flow must have a schedule."
         assert latest_metadata.retention_policy is not None, "A flow must have a retention policy."
 
-        url = generate_ui_url(api_client.__GLOBAL_API_CLIENT__.construct_base_url(), self._id)
+        url = generate_ui_url(globals.__GLOBAL_API_CLIENT__.construct_base_url(), self._id)
 
         print(
             textwrap.dedent(
@@ -172,11 +155,3 @@ class Flow:
             )
         )
         print(json.dumps(self.list_runs(), sort_keys=False, indent=4))
-
-
-class FlowConfig(BaseModel):
-    engine: Optional[AirflowIntegration]
-
-    class Config:
-        # Necessary to allow an engine field
-        arbitrary_types_allowed = True
